
//
//  Quat3DX86.cpp - Quaternions
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>
#include    <math.h>

    //  Book Development Library Modules

#include    "vmp3D.h"
#include    "vmp3DEx.h"
#include    "Matrix3D.h"                                    // 3D Math
#include    "Matrix3DEx.h"
#include    "Quat3D.h"
#include    "Quat3DEx.h"
#include    "QVas3D.h"
#include    "QVas3DEx.h"



//
//  Setup Quaternions module
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void Quat3DSetup( const CpuInfo * const pcinfo )
{
    if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_IsVEqual =              vmp_IsVEqualGeneric;
        vmp_IsQVEqual =             vmp_IsQVEqualAsmSSE;
        vmp_IsMatrixEqual =         vmp_IsMatrixEqualGeneric;   //AsmSSE;

            //  Quat is identical to QVec just different type definition.
        vmp_IsQuatEqual =           vmp_IsQuatEqualGeneric;

        vmp_QuatAdd =               (vmp_QuatAddProc) vmp_QVecAddAsmSSE;
        vmp_QuatSub =               (vmp_QuatSubProc) vmp_QVecSubAsmSSE;

        vmp_QuatCopy =              vmp_QuatCopyGeneric;
        vmp_QuatConjugate =         vmp_QuatConjugateAsmSSE;
        vmp_QuatInv =               vmp_QuatInvGeneric;
//      vmp_QuatInvFast =           vmp_QuatInvGeneric;             // *FAST*
        vmp_QuatSquare =            vmp_QuatSquareGeneric;
        vmp_QuatSqrt =              vmp_QuatSqrtGeneric;
        vmp_QuatDotProduct =        vmp_QuatDotProductAsmSSE;
        vmp_QuatMagnitude =         vmp_QuatMagnitudeAsmSSE;
        vmp_QuatMagnitudeFast =     vmp_QuatMagnitudeFastAsmSSE;    // *FAST*
        vmp_QuatNormalize =         vmp_QuatNormalizeAsmSSE;
//      vmp_QuatNormalizeFast =     vmp_QuatNormalizeAsmSSE;        // *FAST*
        vmp_QuatNeg =               vmp_QuatNegGeneric;
        vmp_QuatRotAxis =           vmp_QuatRotAxisGeneric;
        vmp_QuatToRotAxis =         vmp_QuatToRotAxisGeneric;
        vmp_QuatRotYawPitchRoll =   vmp_QuatRotYawPitchRollGeneric;
        vmp_QuatMul =               vmp_QuatMulAsmSSE;
        vmp_QuatDiv =               vmp_QuatDivGeneric;
        vmp_QuatExp =               vmp_QuatExpGeneric;
        vmp_QuatLn =                vmp_QuatLnGeneric;
        vmp_QuatToMatrix =          vmp_QuatToMatrixGeneric;
        vmp_QuatMatrix =            vmp_QuatMatrixGeneric;
        vmp_QuatSlerp =             vmp_QuatSlerpGeneric;
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_IsVEqual =              vmp_IsVEqualGeneric;
        vmp_IsQVEqual =             vmp_IsQVEqualAsm3DNow;
        vmp_IsMatrixEqual =         vmp_IsMatrixEqualAsm3DNow;

            //  Quat is identical to QVec just different type definition.
        vmp_IsQuatEqual =           vmp_IsQuatEqualGeneric;

        vmp_QuatAdd =               (vmp_QuatAddProc) vmp_QVecAddAsm3DNow;
        vmp_QuatSub =               (vmp_QuatSubProc) vmp_QVecSubAsm3DNow;

        vmp_QuatCopy =              vmp_QuatCopyGeneric;
        vmp_QuatConjugate =         vmp_QuatConjugateAsm3DNow;
        vmp_QuatInv =               vmp_QuatInvAsm3DNow;
//      vmp_QuatInvFast =           vmp_QuatInvGeneric;             // *FAST*
        vmp_QuatSquare =            vmp_QuatSquareGeneric;
        vmp_QuatSqrt =              vmp_QuatSqrtGeneric;
        vmp_QuatDotProduct =        vmp_QuatDotProductAsm3DNow;
        vmp_QuatMagnitude =         vmp_QuatMagnitudeAsm3DNow;
        vmp_QuatMagnitudeFast =     vmp_QuatMagnitudeFastAsm3DNow;  // *FAST*
        vmp_QuatNormalize =         vmp_QuatNormalizeAsm3DNow;
//      vmp_QuatNormalizeFast =     vmp_QuatNormalizeAsm3DNow;      // *FAST*
        vmp_QuatNeg =               vmp_QuatNegGeneric;
        vmp_QuatRotAxis =           vmp_QuatRotAxisGeneric;
        vmp_QuatToRotAxis =         vmp_QuatToRotAxisGeneric;
        vmp_QuatRotYawPitchRoll =   vmp_QuatRotYawPitchRollGeneric;
        vmp_QuatMul =               vmp_QuatMulAsm3DNow;
        vmp_QuatDiv =               vmp_QuatDivGeneric;
        vmp_QuatExp =               vmp_QuatExpGeneric;
        vmp_QuatLn =                vmp_QuatLnGeneric;
        vmp_QuatToMatrix =          vmp_QuatToMatrixGeneric;
        vmp_QuatMatrix =            vmp_QuatMatrixGeneric;
        vmp_QuatSlerp =             vmp_QuatSlerpGeneric;
      }
    
        // Don't use MMX versions, only generic! Prevent EMMS!

    else if ( CPUBITS_MMX & pcinfo->nBits )
      {
        vmp_IsVEqual =              vmp_IsVEqualGeneric;
        vmp_IsQVEqual =             vmp_IsQVEqualGeneric;
        vmp_IsMatrixEqual =         vmp_IsMatrixEqualGeneric;

            //  Quat is identical to QVec just different type definition.
        vmp_IsQuatEqual =           vmp_IsQuatEqualGeneric;

        vmp_QuatAdd =               (vmp_QuatAddProc) vmp_QVecAddGeneric;
        vmp_QuatSub =               (vmp_QuatSubProc) vmp_QVecSubGeneric;

        vmp_QuatCopy =              vmp_QuatCopyGeneric;
        vmp_QuatConjugate =         vmp_QuatConjugateGeneric;
        vmp_QuatInv =               vmp_QuatInvGeneric;
//      vmp_QuatInvFast =           vmp_QuatInvGeneric;             // *FAST*
        vmp_QuatSquare =            vmp_QuatSquareGeneric;
        vmp_QuatSqrt =              vmp_QuatSqrtGeneric;
        vmp_QuatDotProduct =        vmp_QuatDotProductGeneric;
        vmp_QuatMagnitude =         vmp_QuatMagnitudeGeneric;
        vmp_QuatMagnitudeFast =     vmp_QuatMagnitudeGeneric;       // *FAST*
        vmp_QuatNormalize =         vmp_QuatNormalizeGeneric;
//      vmp_QuatNormalizeFast =     vmp_QuatNormalizeGeneric;       // *FAST*
        vmp_QuatNeg =               vmp_QuatNegGeneric;
        vmp_QuatRotAxis =           vmp_QuatRotAxisGeneric;
        vmp_QuatToRotAxis =         vmp_QuatToRotAxisGeneric;
        vmp_QuatRotYawPitchRoll =   vmp_QuatRotYawPitchRollGeneric;
        vmp_QuatMul =               vmp_QuatMulGeneric;
        vmp_QuatDiv =               vmp_QuatDivGeneric;
        vmp_QuatExp =               vmp_QuatExpGeneric;
        vmp_QuatLn =                vmp_QuatLnGeneric;
        vmp_QuatToMatrix =          vmp_QuatToMatrixGeneric;
        vmp_QuatMatrix =            vmp_QuatMatrixGeneric;
        vmp_QuatSlerp =             vmp_QuatSlerpGeneric;
      }

    else
      {
        ASSERT(0);
      }
}


